/*!\file primitives.h
 *
 * \brief Primitives de dessin 2D sur texture GL
 *
 * A but pédagogique, ne gère qu'un seul écran.
 * \author Farès BELHADJ, amsi@ai.univ-paris8.fr
 * \date March 14 2014
 */

#ifndef PRIMITIVES_H

#define PRIMITIVES_H

#include <gl4dummies.h> /* pour le type Uint32 et GLuint */

#if SDL_BYTEORDER == SDL_BIGENDIAN 
  /*!\brief décalage pour la composante rouge */
  #define R_SHIFT 24
  /*!\brief décalage pour la composante verte */
  #define G_SHIFT 16
  /*!\brief décalage pour la composante bleue */
  #define B_SHIFT  8
  /*!\brief décalage pour la composante alpha */
  #define A_SHIFT  0
#else
  /*!\brief décalage pour la composante rouge */
  #define R_SHIFT  0
  /*!\brief décalage pour la composante verte */
  #define G_SHIFT  8
  /*!\brief décalage pour la composante bleue */
  #define B_SHIFT 16
  /*!\brief décalage pour la composante alpha */
  #define A_SHIFT 24
#endif
/*!\brief macro pour tester si x et y sont dans limites de l'ecran */
#define IN_SCREEN(x, y) ((x) >= 0 && (x) < getScreenWidth() && (y) >= 0 && (y) < getScreenHeight())
/*!\brief macro qui convertie un r, un g, un b et un a en couleur Uint32 rgba */
#define RGBA(r, g, b, a) (((Uint32)(Uint8)(r)) << R_SHIFT |	\
			  ((Uint32)(Uint8)(g)) << G_SHIFT |	\
			  ((Uint32)(Uint8)(b)) << B_SHIFT |	\
			  ((Uint32)(Uint8)(a)) << A_SHIFT )
/*!\brief macro qui convertie un r, un g et un b en couleur Uint32 rgba dont l'alpha est à 255 */
#define RGB(r, g, b) RGBA(r, g, b, 255)
/*!\brief macro permettant d'extraire une composante en utilisant l'argument \a shift */
#define EXTRACT_COMP(coul, shift) ((((Uint32)(coul)) >> (shift)) & 0xFF)
/*!\brief macro permettant d'extraire une composante rouge de l'Uint32 \a coul */
#define   RED(coul) EXTRACT_COMP(coul, R_SHIFT)
/*!\brief macro permettant d'extraire une composante verte de l'Uint32 \a coul */
#define GREEN(coul) EXTRACT_COMP(coul, G_SHIFT)
/*!\brief macro permettant d'extraire une composante bleue de l'Uint32 \a coul */
#define  BLUE(coul) EXTRACT_COMP(coul, B_SHIFT)
/*!\brief macro permettant d'extraire une composante alpha de l'Uint32 \a coul */
#define ALPHA(coul) EXTRACT_COMP(coul, A_SHIFT)

#ifdef __cplusplus
extern "C" {
#endif

  extern Uint32 getCurColor(void);
  extern void   setColor32(Uint32 color);
  extern void   initScreen(GLuint tId, GLuint w, GLuint h);
  extern GLuint getScreenWidth(void);
  extern GLuint getScreenHeight(void);
  extern void   clearScreen(Uint32 color);
  extern void   updateScreen(GLuint * rect);
  extern void   deleteScreen(void);
  extern Uint32 getPixel(int x, int y);
  extern void   putPixel(int x, int y);
  extern void   line(int x0, int y0, int x1, int y1);
  extern void   circle(int x0, int y0, int r);
  extern void   hLine(int x0, int x1, int y);
  extern void   filledCircle(int x0, int y0, int r);
  
#ifdef __cplusplus
}
#endif

#endif
